searchData={"items":[{"type":"module","title":"PPlusFireStore","doc":"Documentation for `PPlusFireStore`.\n\nThis module is responsible for initializing the service that will manage access tokens to Google Firestore with the access credentials configured for your Firestore repository module.\n\nBy default, the service responsible for fetching the access token is [Goth](https://github.com/peburrows/goth), but you can configure another access token fetching service through the `:token_fetcher` key in your repository configuration.\n\n[Goth](https://github.com/peburrows/goth) uses the configured access credentials to fetch the access token and cache it. When requested, the service retrieves the access token from the cache and checks if it is still valid. If the access token is not in the cache or has expired, the service will fetch a new access token.","ref":"PPlusFireStore.html"},{"type":"module","title":"Usage - PPlusFireStore","doc":"Define a repository module that implements the PPlusFireStore.Repo interface.\n```elixir\ndefmodule MyFireStoreRepo do\n  use PPlusFireStore.Repo, otp_app: :my_app\nend\n```\n\nConfigure the repository module with Google Firestore credentials and add it to your supervision tree.\n```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  @impl true\n  def start(_type, _args) do\n    children = [\n      {PPlusFireStore, [MyFireStoreRepo]}\n    ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nYou can have more than one configured repository module, just add more repository modules to the list of modules.\n```elixir\nchildren = [\n  {PPlusFireStore, [MyFireStoreRepo, MyOtherFireStoreRepo]}\n]\n```","ref":"PPlusFireStore.html#module-usage"},{"type":"function","title":"PPlusFireStore.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"PPlusFireStore.html#child_spec/1"},{"type":"function","title":"PPlusFireStore.start_link/1","doc":"","ref":"PPlusFireStore.html#start_link/1"},{"type":"module","title":"PPlusFireStore.API","doc":"Module to interact with Google Firestore API","ref":"PPlusFireStore.API.html"},{"type":"function","title":"PPlusFireStore.API.create_document/5","doc":"Create document","ref":"PPlusFireStore.API.html#create_document/5"},{"type":"function","title":"Parameters - PPlusFireStore.API.create_document/5","doc":"- auth_token: auth token\n  - parent: The parent resource. For example: `projects/{project_id}/databases/{database_id}/documents` or `projects/{project_id}/databases/{database_id}/documents/chatrooms/{chatroom_id}`\n  - collection: collection name\n  - data: map with document data\n  - opts: optional parameters. See: https://hexdocs.pm/google_api_firestore/GoogleApi.Firestore.V1.Api.Projects.html#firestore_projects_databases_documents_create_document/5\n\n  Example:\n\n    iex> PPlusFireStore.API.create_document(\"token\", \"projects/my_project/databases/(default)/documents\", \"books\", %{author: \"John Doe\"})\n    {:ok,\n      %PPlusFireStore.Model.Document{\n        path: \"projects/my_project/databases/(default)/documents/books/esgXQM7pqNCwQwYRJeBJ\",\n        data: %{\"author\" => \"John Due\"},\n        created_at: ~U[2025-01-10 17:14:04.738331Z],\n        updated_at: ~U[2025-01-10 17:14:04.738331Z]\n      }}\n\n  Note: One of the optional parameters is `documentId`, which is the id of the document to be created. If not provided, Firestore will generate an id automatically. It is important to pass the document id if you want to control the creation of duplicate documents.","ref":"PPlusFireStore.API.html#create_document/5-parameters"},{"type":"function","title":"PPlusFireStore.API.delete_document/3","doc":"Delete document from firestore","ref":"PPlusFireStore.API.html#delete_document/3"},{"type":"function","title":"Parameters - PPlusFireStore.API.delete_document/3","doc":"- auth_token: auth token\n  - path: The name of the document to delete. For example: `projects/{project_id}/databases/{database_id}/documents/{document_path}`\n  - opts: optional parameters. See: https://hexdocs.pm/google_api_firestore/GoogleApi.Firestore.V1.Api.Projects.html#firestore_projects_databases_documents_delete/4\n\n  Example:\n\n    iex> PPlusFireStore.API.delete_document(\"token\", \"projects/my_project/databases/(default)/documents/books/esgXQM7pqNCwQwYRJeBJ\")\n    {:ok, :deleted}","ref":"PPlusFireStore.API.html#delete_document/3-parameters"},{"type":"type","title":"PPlusFireStore.API.error_code/0","doc":"","ref":"PPlusFireStore.API.html#t:error_code/0"},{"type":"function","title":"PPlusFireStore.API.get_document/3","doc":"Get document from firestore","ref":"PPlusFireStore.API.html#get_document/3"},{"type":"function","title":"Parameters - PPlusFireStore.API.get_document/3","doc":"- auth_token: auth token\n  - path: The name of the document to retrieve. For example: `projects/{project_id}/databases/{database_id}/documents/{document_path}`\n  - opts: optional parameters. See: https://hexdocs.pm/google_api_firestore/GoogleApi.Firestore.V1.Api.Projects.html#firestore_projects_databases_documents_get/4\n\n  Example:\n    iex> PPlusFireStore.API.get_document(\"token\", \"projects/my_project/databases/(default)/documents/books/esgXQM7pqNCwQwYRJeBJ\")\n    {:ok,\n      %PPlusFireStore.Model.Document{\n        path: \"projects/my_project/databases/(default)/documents/books/esgXQM7pqNCwQwYRJeBJ\",\n        data: %{\"author\" => \"John Due\"},\n        created_at: ~U[2025-01-10 17:14:04.738331Z],\n        updated_at: ~U[2025-01-10 17:14:04.738331Z]\n      }}","ref":"PPlusFireStore.API.html#get_document/3-parameters"},{"type":"function","title":"PPlusFireStore.API.list_documents/4","doc":"List documents from firestore","ref":"PPlusFireStore.API.html#list_documents/4"},{"type":"function","title":"Parameters - PPlusFireStore.API.list_documents/4","doc":"- auth_token: auth token\n  - parent: The parent resource. For example: `projects/{project_id}/databases/{database_id}/documents` or `projects/{project_id}/databases/{database_id}/documents/chatrooms/{chatroom_id}`\n  - collection: collection name\n  - opts: optional parameters. See: https://hexdocs.pm/google_api_firestore/GoogleApi.Firestore.V1.Api.Projects.html#firestore_projects_databases_documents_list/5\n\n  Example:\n    iex> PPlusFireStore.API.list_documents(\"token\", \"projects/my_project/databases/(default)/documents\", \"books\", pageSize: 1)\n    {:ok,\n      %PPlusFireStore.Model.Page{\n        data: [\n          %PPlusFireStore.Model.Document{\n            path: \"projects/my_project/databases/(default)/documents/books/esgXQM7pqNCwQwYRJeBJ\",\n            data: %{\"author\" => \"John Due\"},\n            created_at: ~U[2025-01-10 17:14:04.738331Z],\n            updated_at: ~U[2025-01-10 17:14:04.738331Z]\n          }\n        ],\n        next_page_token: \"AFTOeJwGTcAtgJAapbJ0K7tPwpH9saWYfm4bG991Kk4qdP3NXq9pFfp5IW-E6lwbnRW661DKMJjo5EA7y2iF8GFjaCPLlXN7c0jMYATSRgclgLEChgsSIBjt\"\n      }}","ref":"PPlusFireStore.API.html#list_documents/4-parameters"},{"type":"function","title":"PPlusFireStore.API.run_query/4","doc":"","ref":"PPlusFireStore.API.html#run_query/4"},{"type":"function","title":"PPlusFireStore.API.update_document/4","doc":"Update document in firestore","ref":"PPlusFireStore.API.html#update_document/4"},{"type":"function","title":"Parameters - PPlusFireStore.API.update_document/4","doc":"- auth_token: auth token\n  - path: The name of the document to update. For example: `projects/{project_id}/databases/{database_id}/documents/{document_path}`\n  - data: map with document data\n  - opts: optional parameters. See: https://hexdocs.pm/google_api_firestore/GoogleApi.Firestore.V1.Api.Projects.html#firestore_projects_databases_documents_patch/5\n\n  Example:\n\n    iex> PPlusFireStore.API.update_document(\"token\", \"projects/my_project/databases/(default)/documents/books/esgXQM7pqNCwQwYRJeBJ\", %{author: \"John Doe da Silva\"})\n    {:ok,\n      %PPlusFireStore.Model.Document{\n        path: \"projects/my_project/databases/(default)/documents/books/esgXQM7pqNCwQwYRJeBJ\",\n        data: %{\"author\" => \"John Due da Silva\"},\n        created_at: ~U[2025-01-10 17:14:04.738331Z],\n        updated_at: ~U[2025-01-10 17:14:04.738331Z]\n      }}","ref":"PPlusFireStore.API.html#update_document/4-parameters"},{"type":"module","title":"PPlusFireStore.Connection","doc":"Handle connection with Google Firestore API\n\n\nconfiguration:\n\n    config :google_api_firestore, :base_url, \"https://my-custom-url.com/\"","ref":"PPlusFireStore.Connection.html"},{"type":"function","title":"PPlusFireStore.Connection.delete/3","doc":"Perform a DELETE request.\n\nSee `request/1` or `request/2` for options definition.\n\n    delete(\"/users\")\n    delete(\"/users\", query: [scope: \"admin\"])\n    delete(client, \"/users\")\n    delete(client, \"/users\", query: [scope: \"admin\"])\n    delete(client, \"/users\", body: %{name: \"Jon\"})","ref":"PPlusFireStore.Connection.html#delete/3"},{"type":"function","title":"PPlusFireStore.Connection.delete!/3","doc":"Perform a DELETE request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    delete!(\"/users\")\n    delete!(\"/users\", query: [scope: \"admin\"])\n    delete!(client, \"/users\")\n    delete!(client, \"/users\", query: [scope: \"admin\"])\n    delete!(client, \"/users\", body: %{name: \"Jon\"})","ref":"PPlusFireStore.Connection.html#delete!/3"},{"type":"function","title":"PPlusFireStore.Connection.execute/2","doc":"Execute a request on this connection","ref":"PPlusFireStore.Connection.html#execute/2"},{"type":"function","title":"Returns - PPlusFireStore.Connection.execute/2","doc":"*   `{:ok, Tesla.Env.t}` - If the call was successful\n*   `{:error, reason}` - If the call failed","ref":"PPlusFireStore.Connection.html#execute/2-returns"},{"type":"function","title":"PPlusFireStore.Connection.get/3","doc":"Perform a GET request.\n\nSee `request/1` or `request/2` for options definition.\n\n    get(\"/users\")\n    get(\"/users\", query: [scope: \"admin\"])\n    get(client, \"/users\")\n    get(client, \"/users\", query: [scope: \"admin\"])\n    get(client, \"/users\", body: %{name: \"Jon\"})","ref":"PPlusFireStore.Connection.html#get/3"},{"type":"function","title":"PPlusFireStore.Connection.get!/3","doc":"Perform a GET request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    get!(\"/users\")\n    get!(\"/users\", query: [scope: \"admin\"])\n    get!(client, \"/users\")\n    get!(client, \"/users\", query: [scope: \"admin\"])\n    get!(client, \"/users\", body: %{name: \"Jon\"})","ref":"PPlusFireStore.Connection.html#get!/3"},{"type":"function","title":"PPlusFireStore.Connection.head/3","doc":"Perform a HEAD request.\n\nSee `request/1` or `request/2` for options definition.\n\n    head(\"/users\")\n    head(\"/users\", query: [scope: \"admin\"])\n    head(client, \"/users\")\n    head(client, \"/users\", query: [scope: \"admin\"])\n    head(client, \"/users\", body: %{name: \"Jon\"})","ref":"PPlusFireStore.Connection.html#head/3"},{"type":"function","title":"PPlusFireStore.Connection.head!/3","doc":"Perform a HEAD request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    head!(\"/users\")\n    head!(\"/users\", query: [scope: \"admin\"])\n    head!(client, \"/users\")\n    head!(client, \"/users\", query: [scope: \"admin\"])\n    head!(client, \"/users\", body: %{name: \"Jon\"})","ref":"PPlusFireStore.Connection.html#head!/3"},{"type":"function","title":"PPlusFireStore.Connection.new/0","doc":"Configure an authless client connection","ref":"PPlusFireStore.Connection.html#new/0"},{"type":"function","title":"Returns - PPlusFireStore.Connection.new/0","doc":"*   `Tesla.Env.client`","ref":"PPlusFireStore.Connection.html#new/0-returns"},{"type":"function","title":"PPlusFireStore.Connection.new/1","doc":"Configure a client connection using a function which yields a Bearer token.","ref":"PPlusFireStore.Connection.html#new/1"},{"type":"function","title":"Parameters - PPlusFireStore.Connection.new/1","doc":"*   `token_fetcher` (*type:* `list(String.t()) -> String.t()`) - Callback\n    which provides an OAuth2 token given a list of scopes","ref":"PPlusFireStore.Connection.html#new/1-parameters"},{"type":"function","title":"Returns - PPlusFireStore.Connection.new/1","doc":"*   `Tesla.Env.client`","ref":"PPlusFireStore.Connection.html#new/1-returns"},{"type":"type","title":"PPlusFireStore.Connection.option/0","doc":"Options that may be passed to a request function. See `request/2` for detailed descriptions.","ref":"PPlusFireStore.Connection.html#t:option/0"},{"type":"function","title":"PPlusFireStore.Connection.options/3","doc":"Perform a OPTIONS request.\n\nSee `request/1` or `request/2` for options definition.\n\n    options(\"/users\")\n    options(\"/users\", query: [scope: \"admin\"])\n    options(client, \"/users\")\n    options(client, \"/users\", query: [scope: \"admin\"])\n    options(client, \"/users\", body: %{name: \"Jon\"})","ref":"PPlusFireStore.Connection.html#options/3"},{"type":"function","title":"PPlusFireStore.Connection.options!/3","doc":"Perform a OPTIONS request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    options!(\"/users\")\n    options!(\"/users\", query: [scope: \"admin\"])\n    options!(client, \"/users\")\n    options!(client, \"/users\", query: [scope: \"admin\"])\n    options!(client, \"/users\", body: %{name: \"Jon\"})","ref":"PPlusFireStore.Connection.html#options!/3"},{"type":"function","title":"PPlusFireStore.Connection.patch/4","doc":"Perform a PATCH request.\n\nSee `request/1` or `request/2` for options definition.\n\n    patch(\"/users\", %{name: \"Jon\"})\n    patch(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    patch(client, \"/users\", %{name: \"Jon\"})\n    patch(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"PPlusFireStore.Connection.html#patch/4"},{"type":"function","title":"PPlusFireStore.Connection.patch!/4","doc":"Perform a PATCH request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    patch!(\"/users\", %{name: \"Jon\"})\n    patch!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    patch!(client, \"/users\", %{name: \"Jon\"})\n    patch!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"PPlusFireStore.Connection.html#patch!/4"},{"type":"function","title":"PPlusFireStore.Connection.post/4","doc":"Perform a POST request.\n\nSee `request/1` or `request/2` for options definition.\n\n    post(\"/users\", %{name: \"Jon\"})\n    post(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    post(client, \"/users\", %{name: \"Jon\"})\n    post(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"PPlusFireStore.Connection.html#post/4"},{"type":"function","title":"PPlusFireStore.Connection.post!/4","doc":"Perform a POST request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    post!(\"/users\", %{name: \"Jon\"})\n    post!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    post!(client, \"/users\", %{name: \"Jon\"})\n    post!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"PPlusFireStore.Connection.html#post!/4"},{"type":"function","title":"PPlusFireStore.Connection.put/4","doc":"Perform a PUT request.\n\nSee `request/1` or `request/2` for options definition.\n\n    put(\"/users\", %{name: \"Jon\"})\n    put(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    put(client, \"/users\", %{name: \"Jon\"})\n    put(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"PPlusFireStore.Connection.html#put/4"},{"type":"function","title":"PPlusFireStore.Connection.put!/4","doc":"Perform a PUT request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    put!(\"/users\", %{name: \"Jon\"})\n    put!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    put!(client, \"/users\", %{name: \"Jon\"})\n    put!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"PPlusFireStore.Connection.html#put!/4"},{"type":"function","title":"PPlusFireStore.Connection.request/2","doc":"Perform a request.","ref":"PPlusFireStore.Connection.html#request/2"},{"type":"function","title":"Options - PPlusFireStore.Connection.request/2","doc":"- `:method` - the request method, one of [`:head`, `:get`, `:delete`, `:trace`, `:options`, `:post`, `:put`, `:patch`]\n- `:url` - either full url e.g. \"http://example.com/some/path\" or just \"/some/path\" if using `Tesla.Middleware.BaseUrl`\n- `:query` - a keyword list of query params, e.g. `[page: 1, per_page: 100]`\n- `:headers` - a keyword list of headers, e.g. `[{\"content-type\", \"text/plain\"}]`\n- `:body` - depends on used middleware:\n    - by default it can be a binary\n    - if using e.g. JSON encoding middleware it can be a nested map\n    - if adapter supports it it can be a Stream with any of the above\n- `:opts` - custom, per-request middleware or adapter options","ref":"PPlusFireStore.Connection.html#request/2-options"},{"type":"function","title":"Examples - PPlusFireStore.Connection.request/2","doc":"ExampleApi.request(method: :get, url: \"/users/path\")\n\n    # use shortcut methods\n    ExampleApi.get(\"/users/1\")\n    ExampleApi.post(client, \"/users\", %{name: \"Jon\"})","ref":"PPlusFireStore.Connection.html#request/2-examples"},{"type":"function","title":"PPlusFireStore.Connection.request!/2","doc":"Perform request and raise in case of error.\n\nThis is similar to `request/2` behaviour from Tesla 0.x\n\nSee `request/2` for list of available options.","ref":"PPlusFireStore.Connection.html#request!/2"},{"type":"type","title":"PPlusFireStore.Connection.t/0","doc":"","ref":"PPlusFireStore.Connection.html#t:t/0"},{"type":"function","title":"PPlusFireStore.Connection.trace/3","doc":"Perform a TRACE request.\n\nSee `request/1` or `request/2` for options definition.\n\n    trace(\"/users\")\n    trace(\"/users\", query: [scope: \"admin\"])\n    trace(client, \"/users\")\n    trace(client, \"/users\", query: [scope: \"admin\"])\n    trace(client, \"/users\", body: %{name: \"Jon\"})","ref":"PPlusFireStore.Connection.html#trace/3"},{"type":"function","title":"PPlusFireStore.Connection.trace!/3","doc":"Perform a TRACE request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    trace!(\"/users\")\n    trace!(\"/users\", query: [scope: \"admin\"])\n    trace!(client, \"/users\")\n    trace!(client, \"/users\", query: [scope: \"admin\"])\n    trace!(client, \"/users\", body: %{name: \"Jon\"})","ref":"PPlusFireStore.Connection.html#trace!/3"},{"type":"module","title":"PPlusFireStore.Decoder","doc":"Documentation for `PPlusFireStore.Decoder`.\n\nThis module is responsible for decoding the response from Google Firestore API.","ref":"PPlusFireStore.Decoder.html"},{"type":"module","title":"Example - PPlusFireStore.Decoder","doc":"iex> PPlusFireStore.Decoder.decode(%GoogleApi.Firestore.V1.Model.Document{\n    ...>   name: \"projects/my_project/databases/(default)/documents/books/esgXQM7pqNCwQwYRJeBJ\",\n    ...>   fields: %{\n    ...>     \"author\" => %GoogleApi.Firestore.V1.Model.Value{stringValue: \"John Doe\"}\n    ...>   },\n    ...>   createTime: ~U[2025-01-10 17:14:04.738331Z],\n    ...>   updateTime: ~U[2025-01-10 17:14:04.738331Z]\n    ...> })\n    %PPlusFireStore.Model.Document{\n      path: \"projects/my_project/databases/(default)/documents/books/esgXQM7pqNCwQwYRJeBJ\",\n      data: %{\"author\" => \"John Doe\"},\n      created_at: ~U[2025-01-10 17:14:04.738331Z],\n      updated_at: ~U[2025-01-10 17:14:04.738331Z]\n    }","ref":"PPlusFireStore.Decoder.html#module-example"},{"type":"function","title":"PPlusFireStore.Decoder.decode/1","doc":"","ref":"PPlusFireStore.Decoder.html#decode/1"},{"type":"module","title":"PPlusFireStore.Encoder","doc":"Documentation for `PPlusFireStore.Encoder`.\n\nThe Google Firestore API expects a map in a complex format containing the data and their respective types.\n\nThe Encoder module is responsible for encoding the data to be sent.\n\nExample:\n\n```elixir\niex> PPlusFireStore.Encoder.encode(%{\"name\" => \"John Doe\", \"age\" => 42})\n%{\n  fields: %{\n    \"name\" => %{stringValue: \"John Doe\"},\n    \"age\" => %{integerValue: 42}\n  }\n}\n```\n\nThere are also special types such as `DateTime`, `GeoPoint`, and `Reference`.\n\n```elixir\niex> PPlusFireStore.Encoder.encode(%{\"name\" => \"John Doe\", \"age\" => 42, \"birth_data\" => ~U[1998-06-02 09:30:01.023149Z]})\n%{\n  fields: %{\n    \"age\" => %{integerValue: 42},\n    \"birth_data\" => %{timestampValue: ~U[1998-06-02 09:30:01.023149Z]},\n    \"name\" => %{stringValue: \"John Doe\"}\n  }\n}\n```\n\n```elixir\niex> PPlusFireStore.Encoder.encode(%{\"location\" => {:geo, {5.96989, 31.19063}}})\n%{\n  fields: %{\n    \"location\" => %{geoPointValue: %{latitude: 5.96989, longitude: 31.19063}}\n  }\n}\n```\n\n```elixir\niex> PPlusFireStore.Encoder.encode(%{\"location\" => %{latitude: 5.96989, longitude: 31.19063}})\n%{\n  fields: %{\n    \"location\" => %{geoPointValue: %{latitude: 5.96989, longitude: 31.19063}}\n  }\n}\n```\n\n```elixir\niex> PPlusFireStore.Encoder.encode(%{\"next\" => {:ref, \"projects/my-project/databases/(default)/documents/my-collection/my-document\"}})\n%{\n  fields: %{\n    \"next\" => %{referenceValue: \"projects/my-project/databases/(default)/documents/my-collection/my-document\"}\n  }\n}\n```","ref":"PPlusFireStore.Encoder.html"},{"type":"function","title":"PPlusFireStore.Encoder.encode/1","doc":"","ref":"PPlusFireStore.Encoder.html#encode/1"},{"type":"module","title":"PPlusFireStore.Model","doc":"This module defines structs adapted from the Models of the Google Firestore API","ref":"PPlusFireStore.Model.html"},{"type":"module","title":"PPlusFireStore.Model.Document","doc":"Struct adapted from the Google Firestore API Document model","ref":"PPlusFireStore.Model.Document.html"},{"type":"module","title":"Fields - PPlusFireStore.Model.Document","doc":"- path: The resource name of the document. For example: `projects/{project_id}/databases/{database_id}/documents/{document_path}`\n  - data: The document fields and values. For example: `%{\"author\" => \"John Doe\"}`\n  - created_at: The time at which the document was created. Automatically set by Firestore.\n  - updated_at: The time at which the document was last updated. Automatically set by Firestore.","ref":"PPlusFireStore.Model.Document.html#module-fields"},{"type":"type","title":"PPlusFireStore.Model.Document.t/0","doc":"","ref":"PPlusFireStore.Model.Document.html#t:t/0"},{"type":"module","title":"PPlusFireStore.Model.Page","doc":"Struct adapted from the Google Firestore API documents list response","ref":"PPlusFireStore.Model.Page.html"},{"type":"module","title":"Fields - PPlusFireStore.Model.Page","doc":"- data: The list of documents returned.\n  - next_page_token: The next page token. If not set, there are no subsequent pages.","ref":"PPlusFireStore.Model.Page.html#module-fields"},{"type":"type","title":"PPlusFireStore.Model.Page.t/1","doc":"","ref":"PPlusFireStore.Model.Page.html#t:t/1"},{"type":"module","title":"PPlusFireStore.Query","doc":"Create Firestore queries using Elixir syntax.\n\nThe operations supported by Firestore are:\n- `==` (EQUALS)\n- `!=` (NOT_EQUALS)\n- `>` (GREATER_THAN)\n- `<` (LESS_THAN)\n- `>=` (GREATER_THAN_OR_EQUAL)\n- `<=` (LESS_THAN_OR_EQUAL)\n- `in` (IN)\n- `not in` (NOT_IN)\n- `contains` (ARRAY_CONTAINS)\n- `contains_any` (ARRAY_CONTAINS_ANY)\n- `is_nil` (IS_NULL)","ref":"PPlusFireStore.Query.html"},{"type":"module","title":"Example - PPlusFireStore.Query","doc":"Get from \"books\" filtered by author and ordered by author name in ascending order:\n```elixir\nfrom(\"books\")\n|> where(\"author\" == \"John Doe\")\n|> order_by(\"author\", :asc)\n|> limit(5)\n|> offset(10)\n```\n\nNOTE: Although it is possible to paginate with `limit` and `offset`, the query result does not inform the total number of documents in the collection or if there is a next page.\n\n\nGet from \"books\" filtered by author and ordered by author name in ascending order:\n```elixir\nfrom(\"books\")\n|> where(\"author\" in [\"John Doe\", \"Alice Smith\", \"Bob Brown\"])\n|> order_by(\"author\", :asc)\n```\n\nWhen a field is an array, you can use the `contains` operator to filter by the array content:\n```elixir\nfrom(\"books\")\n|> where(contains(\"tags\", \"fiction\") and \"pages\" > 300)\n```\n\nYou can also use the `contains_any` operator to filter by any of the values in the array:\n```elixir\nfrom(\"books\")\n|> where(contains_any(\"tags\", [\"fiction\", \"non-fiction\"]))\n```\n\nYou can also pass the values dynamically:\n```elixir\ncollection = \"books\"\nfield = \"pages\"\nvalue = 300\n\nquery =\n  from(collection)\n  |> where(field > value)\n```","ref":"PPlusFireStore.Query.html#module-example"},{"type":"function","title":"PPlusFireStore.Query.build_where/3","doc":"","ref":"PPlusFireStore.Query.html#build_where/3"},{"type":"function","title":"PPlusFireStore.Query.build_where/4","doc":"","ref":"PPlusFireStore.Query.html#build_where/4"},{"type":"macro","title":"PPlusFireStore.Query.from/2","doc":"Create a query from a collection with options","ref":"PPlusFireStore.Query.html#from/2"},{"type":"macro","title":"Example - PPlusFireStore.Query.from/2","doc":"```elixir\nfrom(\"books\", where: \"author\" == \"John Doe\", limit: 5, order_by: {\"author\", :asc})\n```\n\n```elixir\nfrom(\"books\", all_descendants: true)\n```","ref":"PPlusFireStore.Query.html#from/2-example"},{"type":"function","title":"PPlusFireStore.Query.limit/2","doc":"Define quantity of documents to return","ref":"PPlusFireStore.Query.html#limit/2"},{"type":"function","title":"Example - PPlusFireStore.Query.limit/2","doc":"```elixir\nfrom(\"books\")\n|> limit(5)\n```","ref":"PPlusFireStore.Query.html#limit/2-example"},{"type":"function","title":"PPlusFireStore.Query.offset/2","doc":"Define the offset of the query","ref":"PPlusFireStore.Query.html#offset/2"},{"type":"function","title":"Example - PPlusFireStore.Query.offset/2","doc":"```elixir\nfrom(\"books\")\n|> limit(5)\n|> offset(10)\n```","ref":"PPlusFireStore.Query.html#offset/2-example"},{"type":"macro","title":"PPlusFireStore.Query.or_where/2","doc":"Add an `OR` filter to the query","ref":"PPlusFireStore.Query.html#or_where/2"},{"type":"macro","title":"Example - PPlusFireStore.Query.or_where/2","doc":"```elixir\nfrom(\"books\")\n|> where(\"author\" == \"John Doe\" and \"pages\" > 300)\n|> or_where(\"author\" == \"Alice Smith\" and \"pages\" <= 200)\n```\nEssa consulta é equivalente a:\n```\nSELECT * FROM books\nWHERE (author = \"John Doe\" AND pages > 300) OR (author = \"Alice Smith\" AND pages <= 200)\n```\n\nAssim como where, você pode encadear vários or_where.\n```elixir\nfrom(\"books\")\n|> where(\"author\" == \"John Doe\" and \"pages\" > 300)\n|> or_where(\"author\" == \"Alice Smith\" and \"pages\" <= 200)\n|> or_where(\"author\" == \"Bob Brown\" and \"metadata.year\" <= 2021)\n|> or_where(\"author\" == \"Charlie Brown\" and \"metadata.publisher\" == \"Publisher A\")\n\n```","ref":"PPlusFireStore.Query.html#or_where/2-example"},{"type":"function","title":"PPlusFireStore.Query.order_by/2","doc":"","ref":"PPlusFireStore.Query.html#order_by/2"},{"type":"function","title":"PPlusFireStore.Query.order_by/3","doc":"Orders the query by one or more fields in ascending or descending order","ref":"PPlusFireStore.Query.html#order_by/3"},{"type":"function","title":"Example - PPlusFireStore.Query.order_by/3","doc":"```elixir\nfrom(\"books\")\n|> order_by(\"author\", :asc)\n```\n\n```elixir\nfrom(\"books\")\n|> order_by([\"author\", \"metadata.year\"], :desc)\n```","ref":"PPlusFireStore.Query.html#order_by/3-example"},{"type":"function","title":"PPlusFireStore.Query.select/2","doc":"Selects the fields to return in the query","ref":"PPlusFireStore.Query.html#select/2"},{"type":"function","title":"Example - PPlusFireStore.Query.select/2","doc":"```elixir\nfrom(\"books\")\n|> select([\"author\", \"metadata.publisher\"])\n```","ref":"PPlusFireStore.Query.html#select/2-example"},{"type":"macro","title":"PPlusFireStore.Query.where/3","doc":"Add a filter to the query","ref":"PPlusFireStore.Query.html#where/3"},{"type":"macro","title":"Example - PPlusFireStore.Query.where/3","doc":"```elixir\nfrom(\"books\")\n|> where(\"author\" == \"John Doe\")\n```\n\n```elixir\nfrom(\"books\")\n|> where(\"author\" == \"John Doe\" and \"pages\" > 300)\n```\n\nMultiple chained filters are equivalent to an `AND`.\n```elixir\nfrom(\"books\")\n|> where(\"author\" in [\"John Doe\", \"Alice Smith\", \"Bob Brown\"])\n|> where(\"pages\" > 300)\n|> where(\"metadata.year\" <= 2021)\n```","ref":"PPlusFireStore.Query.html#where/3-example"},{"type":"behaviour","title":"PPlusFireStore.Repo","doc":"Documentation for `PPlusFireStore.Repo`.\n\nPPlusFireStore.Repo is a simplified API to interact with Google Firestore.\n\nEach repository refers to a specific project and database.","ref":"PPlusFireStore.Repo.html"},{"type":"behaviour","title":"Usage - PPlusFireStore.Repo","doc":"defmodule MyFireStoreRepo do\n      use PPlusFireStore.Repo, otp_app: :my_app\n    end","ref":"PPlusFireStore.Repo.html#module-usage"},{"type":"behaviour","title":"Configuration - PPlusFireStore.Repo","doc":"config :my_app, MyFireStoreRepo,\n      project_id: \"project-fake-123-xy-9zz\",\n      private_key_id: \"abc123def456ghi789jkl012mno345pqr678stu901vwx234yz\",\n      private_key: \"-----BEGIN PRIVATE KEY-----\\nMIIEvQIBAFAKEPRIVATEKEYwTDSERHFwLqESDgLKFDWFLRHSDFo5LKJDFA\\nAKDJFSLkfj3+4LKFJSDklsjfdlskJFLDSJKLFJSDFLJSDFLKJSDfljSDLJF\\n-----END PRIVATE KEY-----\\n\",\n      client_email: \"firebase-adminsdk-fake@project-fake-123-xy-9zz.iam.gserviceaccount.com\",\n      client_id: \"123456789012345678901\",\n      auth_uri: \"https://accounts.google.com/o/oauth2/auth\",\n      token_uri: \"https://oauth2.googleapis.com/token\",\n      auth_provider_x509_cert_url: \"https://www.googleapis.com/oauth2/v1/certs\",\n      client_x509_cert_url: \"https://www.googleapis.com/robot/v1/metadata/x509/firebase-adminsdk-fake%40project-fake-123-xy-9zz.iam.gserviceaccount.com\"\n      database_id: \"(default)\"\n\nBy default `database_id` is \"(default)\", so it is optional.\n\nYou can have a repository for each project and database you want to interact with.","ref":"PPlusFireStore.Repo.html#module-configuration"},{"type":"behaviour","title":"Examples - PPlusFireStore.Repo","doc":"defmodule MyFireStoreRepo do\n      use PPlusFireStore.Repo, otp_app: :my_app\n    end\n\n    defmodule MyOtherFireStoreRepo do\n      use PPlusFireStore.Repo, otp_app: :my_app\n    end","ref":"PPlusFireStore.Repo.html#module-examples"},{"type":"callback","title":"PPlusFireStore.Repo.config/0","doc":"Return firestore repo credentials","ref":"PPlusFireStore.Repo.html#c:config/0"},{"type":"callback","title":"Example - PPlusFireStore.Repo.config/0","doc":"iex> PPlusFireStore.Repo.config()\n    [\n      project_id: \"project-fake-123-xy-9zz\",\n      private_key_id: \"abc123def456ghi789jkl012mno345pqr678stu901vwx234yz\",\n      private_key: \"-----BEGIN PRIVATE KEY-----\nMIIEvQIBAFAKEPRIVATEKEYwTDSERHFwLqESDgLKFDWFLRHSDFo5LKJDFA\nAKDJFSLkfj3+4LKFJSDklsjfdlskJFLDSJKLFJSDFLJSDFLKJSDfljSDLJF\n-----END PRIVATE KEY-----\n\",\n      client_email: \"firebase-adminsdk-fake@project-fake-123-xy-9zz.iam.gserviceaccount.com\",\n      client_id: \"123456789012345678901\",\n      auth_uri: \"https://accounts.google.com/o/oauth2/auth\",\n      token_uri: \"https://oauth2.googleapis.com/token\",\n      auth_provider_x509_cert_url: \"https://www.googleapis.com/oauth2/v1/certs\",\n      client_x509_cert_url: \"https://www.googleapis.com/robot/v1/metadata/x509/firebase-adminsdk-fake%40project-fake-123-xy-9zz.iam.gserviceaccount.com\"\n    ]","ref":"PPlusFireStore.Repo.html#c:config/0-example"},{"type":"callback","title":"PPlusFireStore.Repo.create_document/3","doc":"Create document in firestore","ref":"PPlusFireStore.Repo.html#c:create_document/3"},{"type":"callback","title":"Parameters - PPlusFireStore.Repo.create_document/3","doc":"- path: path to document:\n    - example: \"books\" or \"projects/project_id/databases/(default)/documents/books\"\n  - data: document data\n  - opts: optional parameters. See: https://hexdocs.pm/google_api_firestore/GoogleApi.Firestore.V1.Api.Projects.html#firestore_projects_databases_documents_create_document/5\n\n  Example:\n\n    iex> PPlusFireStore.Repo.create_document(\"books/esgXQM7pqNCwQwYRJeBJ\", %{author: \"John Doe\"})\n    {:ok,\n      %PPlusFireStore.Model.Document{\n        path: \"projects/my_project/databases/(default)/documents/books/esgXQM7pqNCwQwYRJeBJ\",\n        data: %{\"author\" => \"John Due\"},\n        created_at: ~U[2025-01-10 17:14:04.738331Z],\n        updated_at: ~U[2025-01-10 17:14:04.738331Z]\n      }}","ref":"PPlusFireStore.Repo.html#c:create_document/3-parameters"},{"type":"callback","title":"PPlusFireStore.Repo.delete_document/2","doc":"Delete document from firestore","ref":"PPlusFireStore.Repo.html#c:delete_document/2"},{"type":"callback","title":"Parameters - PPlusFireStore.Repo.delete_document/2","doc":"- path: path to document:\n    - example: \"books/esgXQM7pqNCwQwYRJeBJ\" or \"projects/project_id/databases/(default)/documents/books/esgXQM7pqNCwQwYRJeBJ\"\n  - opts: optional parameters. See: https://hexdocs.pm/google_api_firestore/GoogleApi.Firestore.V1.Api.Projects.html#firestore_projects_databases_documents_delete/4\n\n  Example:\n\n    iex> PPlusFireStore.Repo.delete_document(\"books/esgXQM7pqNCwQwYRJeBJ\")\n    {:ok, :deleted}","ref":"PPlusFireStore.Repo.html#c:delete_document/2-parameters"},{"type":"callback","title":"PPlusFireStore.Repo.get_document/2","doc":"Get document from firestore","ref":"PPlusFireStore.Repo.html#c:get_document/2"},{"type":"callback","title":"Parameters - PPlusFireStore.Repo.get_document/2","doc":"- path: path to document:\n    - example: \"books/esgXQM7pqNCwQwYRJeBJ\" or \"projects/project_id/databases/(default)/documents/books/esgXQM7pqNCwQwYRJeBJ\"\n  - opts: optional parameters. See: https://hexdocs.pm/google_api_firestore/GoogleApi.Firestore.V1.Api.Projects.html#firestore_projects_databases_documents_get/4\n\n  Example:\n\n    iex> PPlusFireStore.Repo.get_document(\"books/esgXQM7pqNCwQwYRJeBJ\")\n    {:ok,\n      %PPlusFireStore.Model.Document{\n        path: \"projects/my_project/databases/(default)/documents/books/esgXQM7pqNCwQwYRJeBJ\",\n        data: %{\"author\" => \"John Due\"},\n        created_at: ~U[2025-01-10 17:14:04.738331Z],\n        updated_at: ~U[2025-01-10 17:14:04.738331Z]\n      }}","ref":"PPlusFireStore.Repo.html#c:get_document/2-parameters"},{"type":"callback","title":"PPlusFireStore.Repo.list_documents/2","doc":"List documents from firestore","ref":"PPlusFireStore.Repo.html#c:list_documents/2"},{"type":"callback","title":"Parameters - PPlusFireStore.Repo.list_documents/2","doc":"- path: path to document:\n    - example: \"books\" or \"projects/project_id/databases/(default)/documents/books\"\n  - opts: optional parameters. See: https://hexdocs.pm/google_api_firestore/GoogleApi.Firestore.V1.Api.Projects.html#firestore_projects_databases_documents_list/5\n\n  Examples:\n\n    iex> PPlusFireStore.Repo.list_documents(\"books\", pageSize: 1)\n    {:ok,\n      %PPlusFireStore.Model.Page{\n        data: [\n          %PPlusFireStore.Model.Document{\n            path: \"projects/my_project/databases/(default)/documents/books/esgXQM7pqNCwQwYRJeBJ\",\n            data: %{\"author\" => \"John Due\"},\n            created_at: ~U[2025-01-10 17:14:04.738331Z],\n            updated_at: ~U[2025-01-10 17:14:04.738331Z]\n          }\n        ],\n        next_page_token: \"AFTOeJwGTcAtgJAapbJ0K7tPwpH9saWYfm4bG991Kk4qdP3NXq9pFfp5IW-E6lwbnRW661DKMJjo5EA7y2iF8GFjaCPLlXN7c0jMYATSRgclgLEChgsSIBjt\"\n      }}\n\n    iex> PPlusFireStore.Repo.list_documents(\"books\", pageSize: 1, pageToken: \"AFTOeJwGTcAtgJAapbJ0K7tPwpH9saWYfm4bG991Kk4qdP3NXq9pFfp5IW-E6lwbnRW661DKMJjo5EA7y2iF8GFjaCPLlXN7c0jMYATSRgclgLEChgsSIBjt\")\n    {:ok,\n      %PPlusFireStore.Model.Page{\n        data: [\n          %PPlusFireStore.Model.Document{\n            path: \"projects/my_project/databases/(default)/documents/books/esgXQM7pqNCwQwYRJeBJ\",\n            data: %{\"author\" => \"Another John Due\"},\n            created_at: ~U[2025-01-10 17:14:04.738331Z],\n            updated_at: ~U[2025-01-10 17:14:04.738331Z]\n          }\n        ],\n        next_page_token: \"AFTOeJwZgHAqLXB5MfQy9RdN8KJ7vYWLn8KdGpM9X1EqMf2RJwL9Y5JrN3GpJwT4XKuT93HLKDZJg5LP8YiJ8QWnhCVLW7f5Tg2MRB7RKCwFgNJKgQ1VB\"\n      }}","ref":"PPlusFireStore.Repo.html#c:list_documents/2-parameters"},{"type":"callback","title":"PPlusFireStore.Repo.token/0","doc":"Return firestore repo token","ref":"PPlusFireStore.Repo.html#c:token/0"},{"type":"callback","title":"Example - PPlusFireStore.Repo.token/0","doc":"iex> PPlusFireStore.Repo.token\n    \"ya29.c.c0FAKE0TOKEN1234ABC567DEFGHIJKLMNOPQR890STUVWXYZabcd567efghijklmno123pqrstuvw456xyz789ABCDEFGHIJKLMNOpqrstuvWXYz1234567890ABCDEFabcdefGHIJKLMNO98765PQRST4321UVWXYZabc123def456ghi789jkl012mno345pqr678stu901vwx234yzABCDEF123456GHIJKLMNOPQ789RSTUVWXYZabc123def456ghi789jkl012mno345pqr678stu901vwx234yz\"","ref":"PPlusFireStore.Repo.html#c:token/0-example"},{"type":"callback","title":"PPlusFireStore.Repo.update_document/3","doc":"Update document in firestore","ref":"PPlusFireStore.Repo.html#c:update_document/3"},{"type":"callback","title":"Parameters - PPlusFireStore.Repo.update_document/3","doc":"- path: path to document:\n    - example: \"books/esgXQM7pqNCwQwYRJeBJ\" or \"projects/project_id/databases/(default)/documents/books/esgXQM7pqNCwQwYRJeBJ\"\n  - data: document data\n  - opts: optional parameters. See: https://hexdocs.pm/google_api_firestore/GoogleApi.Firestore.V1.Api.Projects.html#firestore_projects_databases_documents_update_document/5\n\n  Example:\n\n    iex> PPlusFireStore.Repo.update_document(\"books/esgXQM7pqNCwQwYRJeBJ\", %{author: \"John Doe da Silva\"})\n    {:ok,\n      %PPlusFireStore.Model.Document{\n        path: \"projects/my_project/databases/(default)/documents/books/esgXQM7pqNCwQwYRJeBJ\",\n        data: %{\"author\" => \"John Due da Silva\"},\n        created_at: ~U[2025-01-10 17:14:04.738331Z],\n        updated_at: ~U[2025-01-10 17:14:04.738331Z]\n      }}","ref":"PPlusFireStore.Repo.html#c:update_document/3-parameters"},{"type":"behaviour","title":"PPlusFireStore.TokenFetcher","doc":"Interface to implement a token fetcher service for Google Firestore access tokens.\n\nThe token fetcher service is responsible for fetching and caching access tokens to Google Firestore.\n\nThe token fetcher service must implement the `fetch/1` callback that receives the module of the repository that is requesting the access token.\n\nSee the `PPlusFireStore.TokenFetcherMock` module for an example implementation of a token fetcher service.","ref":"PPlusFireStore.TokenFetcher.html"},{"type":"callback","title":"PPlusFireStore.TokenFetcher.fetch/1","doc":"","ref":"PPlusFireStore.TokenFetcher.html#c:fetch/1"},{"type":"module","title":"PPlusFireStore.TokenFetcherMock","doc":"Mock module to fetch auth token in tests","ref":"PPlusFireStore.TokenFetcherMock.html"},{"type":"function","title":"PPlusFireStore.TokenFetcherMock.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"PPlusFireStore.TokenFetcherMock.html#child_spec/1"},{"type":"function","title":"PPlusFireStore.TokenFetcherMock.fetch/1","doc":"","ref":"PPlusFireStore.TokenFetcherMock.html#fetch/1"},{"type":"function","title":"PPlusFireStore.TokenFetcherMock.init/1","doc":"","ref":"PPlusFireStore.TokenFetcherMock.html#init/1"},{"type":"function","title":"PPlusFireStore.TokenFetcherMock.start_link/1","doc":"","ref":"PPlusFireStore.TokenFetcherMock.html#start_link/1"},{"type":"extras","title":"README","doc":"---\n\nProvides a simple interface to interact with the official Google API [GoogleApi.Firestore](https://github.com/googleapis/elixir-google-api/tree/master/clients/firestore). It also automatically manages access tokens for Google Firestore.","ref":"readme.html"},{"type":"extras","title":"Installation - README","doc":"```elixir\ndef deps do\n  [\n    {:pplus_firestore, \"~> 0.1.5\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Get firestore credentials - README","doc":"After creating your project in Firestore, go to `Project Settings` > `Service Accounts` > `Generate New Private Key` or use the link https://console.firebase.google.com/u/0/project/_/settings/serviceaccounts/adminsdk\n\nYou will receive something like this:\n\n```json\n{\n  \"type\": \"service_account\",\n  \"project_id\": \"test-project-12345\",\n  \"private_key_id\": \"abc123def456ghi789jkl012mno345pqr678stu901vwx234\",\n  \"private_key\": \"-----BEGIN PRIVATE KEY-----\\nMIIEvAIBADANBgkqhkiG9w0BAQEFAASCBKcwggSjAgEAAoIBAQD123fakeprivatekey987\\n1234567890ABCDEFGHIJKLMN\\n-----END PRIVATE KEY-----\\n\",\n  \"client_email\": \"firebase-adminsdk@test-project-12345.iam.gserviceaccount.com\",\n  \"client_id\": \"123456789012345678901\",\n  \"auth_uri\": \"https://accounts.google.com/o/oauth2/auth\",\n  \"token_uri\": \"https://oauth2.googleapis.com/token\",\n  \"auth_provider_x509_cert_url\": \"https://www.googleapis.com/oauth2/v1/certs\",\n  \"client_x509_cert_url\": \"https://www.googleapis.com/robot/v1/metadata/x509/firebase-adminsdk%40test-project-12345.iam.gserviceaccount.com\",\n  \"universe_domain\": \"googleapis.com\"\n}\n```","ref":"readme.html#get-firestore-credentials"},{"type":"extras","title":"Usage - README","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Create a repository module - README","doc":"```elixir\ndefmodule MyApp.MyFireStoreRepo do\n  use PPlusFireStore.Repo, otp_app: :my_app\nend\n```","ref":"readme.html#create-a-repository-module"},{"type":"extras","title":"Configure your project credentials - README","doc":"```elixir\n# config/config.exs\nconfig :my_app, MyApp.MyFireStoreRepo,\n  project_id: System.fetch_env!(\"FIRESTORE_PROJECT_ID\"),\n  private_key_id: System.fetch_env!(\"FIRESTORE_PRIVATE_KEY_ID\"),\n  private_key: System.fetch_env!(\"FIRESTORE_PRIVATE_KEY\"),\n  client_email: System.fetch_env!(\"FIRESTORE_CLIENT_EMAIL\"),\n  client_id: System.fetch_env!(\"FIRESTORE_CLIENT_ID\"),\n  auth_uri: System.fetch_env!(\"FIRESTORE_AUTH_URI\"),\n  token_uri: System.fetch_env!(\"FIRESTORE_TOKEN_URI\"),\n  auth_provider_x509_cert_url: System.fetch_env!(\"FIRESTORE_AUTH_PROVIDER_CERT_URL\"),\n  client_x509_cert_url: fetch_env!(\"FIRESTORE_CLIENT_CERT_URL\"),\n  database_id: System.get_env(\"FIRESTORE_DATABASE_ID\") # optional, default: \"(default)\"\n```","ref":"readme.html#configure-your-project-credentials"},{"type":"extras","title":"Start PPlusFireStore service in your application - README","doc":"PPlusFireStore will initialize the service that will manage access tokens to the repositories.\nBy default, PPlusFireStore uses [Goth](https://github.com/peburrows/goth) to manage access tokens to the repositories.\nGoth uses the project credentials to generate access tokens for the repositories and stores them in a cache.\nWhen requested, the service retrieves the access tokens from the cache and checks if they are still valid.\nIf the access tokens are not in the cache or are expired, Goth will fetch new access tokens and store them in the cache.\n\nIf you do not want to use Goth, you can implement a custom [TokenFetcher](https://hexdocs.pm/pplus_firestore/PPlusFireStore.TokenFetcher.html) service.\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  @impl true\n  def start(_type, _args) do\n    children = [\n      {PPlusFireStore, [MyApp.MyFireStoreRepo]}\n    ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nPPlusFireStore accepts a list of repositories as an argument. Each repository will have its own credential configuration. Goth will generate and store a different access token for each repository.\n\n```elixir\nchildren = [\n  {PPlusFireStore, [MyApp.MyFireStoreRepo, MyApp.MyOtherFireStoreRepo]}\n]\n```\n\n**Note:** You do not need to initialize this service if you do not want to. You can override the `token/0` function in your repository module to return the token in a different way without using this service.","ref":"readme.html#start-pplusfirestore-service-in-your-application"},{"type":"extras","title":"Examples - README","doc":"","ref":"readme.html#examples"},{"type":"extras","title":"Create a document - README","doc":"The `create_document/3` function provides a simple way to create your documents in Firestore.\n\nYour repository module is already configured with the base path and the database ID.\n\n`projects/ /databases/ /documents`\n\nSo you only need to specify which collection you want to create the document in.\n\n```elixir\niex> MyApp.MyFireStoreRepo.create_document(\"addresses\", %{\n...>   street: \"123 Main St\",\n...>   city: \"Anytown\",\n...>   state: \"CA\",\n...>   postal_code: \"12345\",\n...>   country: \"USA\",\n...>   coordinates: %{\n...>     latitude: 37.7749,\n...>     longitude: -122.4194\n...>   },\n...>   additional_info: %{\n...>     is_residential: true,\n...>     delivery_instructions: nil,\n...>     contact_number: \"+1-555-555-5555\",\n...>     last_verified: ~U[2025-01-10 17:14:04.738331Z]\n...>   },\n...>   created_at: ~U[2025-01-10 17:14:04.738331Z],\n...>   delivery_attempts: 3,\n...>   delivery_notes: [\"Leave at the front door\", \"Ring the bell\"],\n...>   document_reference:\n...>     {:ref, \"projects/my_project/databases/my_database/documents/collection1/doccument1\"}\n...> })\n{:ok,\n %PPlusFireStore.Model.Document{\n   path: \"projects/my_project/databases/my_database/documents/addresses/DxN4EvMyWCh7oTfVmxP9\",\n   data: %{\n     \"additional_info\" => %{\n       \"contact_number\" => \"+1-555-555-5555\",\n       \"delivery_instructions\" => nil,\n       \"is_residential\" => true,\n       \"last_verified\" => ~U[2025-01-10 17:14:04.738331Z]\n     },\n     \"city\" => \"Anytown\",\n     \"coordinates\" => %{\"latitude\" => 37.7749, \"longitude\" => -122.4194},\n     \"country\" => \"USA\",\n     \"created_at\" => ~U[2025-01-10 17:14:04.738331Z],\n     \"delivery_attempts\" => 3,\n     \"delivery_notes\" => [\"Leave at the front door\", \"Ring the bell\"],\n     \"document_reference\" => \"projects/my_project/databases/my_database/documents/collection1/doccument1\",\n     \"postal_code\" => \"12345\",\n     \"state\" => \"CA\",\n     \"street\" => \"123 Main St\"\n   },\n   created_at: ~U[2025-01-20 21:23:18.762968Z],\n   updated_at: ~U[2025-01-20 21:23:18.762968Z]\n }}\n```\n\nThere are 3 types that are special:\n\n- `:ref` - Reference to an existing document.\n- `:geo` - GeoPoint\n- `:bytes` - Bytes\n\nYou can pass them in a map using tuples:\n\n```elixir\n# Reference\n{:ref, \"projects/my_project/databases/my_database/documents/collection1/doccument1\"}\n\n# GeoPoint - Can be 3 ways\n{:geo, {37.7749, -122.4194}}\n{:geo, %{latitude: 37.7749, longitude: -122.4194}}\n%{latitude: 37.7749, longitude: -122.4194}\n\n# Bytes\n{:bytes, \"aGVsbG8gd29ybGQ=\"}\n```","ref":"readme.html#create-a-document"},{"type":"extras","title":"Read a document - README","doc":"The `get_document/2` function provides a simple way to read your documents in Firestore.\n\nYour repository module is already configured with the base path and the database ID.\n\n`projects/ /databases/ /documents`\n\nSo you only need to specify which collection you want to read from and the document ID.\n\n```elixir\niex> MyApp.MyFireStoreRepo.get_document(\"addresses/DxN4EvMyWCh7oTfVmxP9\")\n{:ok,\n %PPlusFireStore.Model.Document{\n   path: \"projects/my_project/databases/my_database/documents/addresses/DxN4EvMyWCh7oTfVmxP9\",\n   data: %{\n     \"additional_info\" => %{\n       \"contact_number\" => \"+1-555-555-5555\",\n       \"delivery_instructions\" => nil,\n       \"is_residential\" => true,\n       \"last_verified\" => ~U[2025-01-10 17:14:04.738331Z]\n     },\n     \"city\" => \"Anytown\",\n     \"coordinates\" => %{\"latitude\" => 37.7749, \"longitude\" => -122.4194},\n     \"country\" => \"USA\",\n     \"created_at\" => ~U[2025-01-10 17:14:04.738331Z],\n     \"delivery_attempts\" => 3,\n     \"delivery_notes\" => [\"Leave at the front door\", \"Ring the bell\"],\n     \"postal_code\" => \"12345\",\n     \"state\" => \"CA\",\n     \"street\" => \"123 Main St\"\n   },\n   created_at: ~U[2025-01-20 21:23:18.762968Z],\n   updated_at: ~U[2025-01-20 21:23:18.762968Z]\n }}\n```\n\nSince a document can store a reference to another document, and that reference contains the full path, you can also pass the full path to `get_document/2`.\n\n```elixir\nMyApp.MyFireStoreRepo.get_document(\"projects/my_project/databases/my_database/documents/addresses/DxN4EvMyWCh7oTfVmxP9\")\n```","ref":"readme.html#read-a-document"},{"type":"extras","title":"Update a document - README","doc":"The `update_document/3` function provides a simple way to update your documents in Firestore.\n\nYour repository module is already configured with the base path and the database ID.\n\n`projects/ /databases/ /documents`\n\nSo you only need to specify the path within the collection and the updated data.\n\n```elixir\niex> MyApp.MyFireStoreRepo.update_document(\"addresses/DxN4EvMyWCh7oTfVmxP9\", %{\n...>   street: \"123 Main St\",\n...>   city: \"Anytown\",\n...>   state: \"CA\",\n...>   postal_code: \"12345\",\n...>   country: \"USA\",\n...>   coordinates: %{\n...>     latitude: 37.7749,\n...>     longitude: -122.4194\n...>   },\n...>   additional_info: %{\n...>     is_residential: true,\n...>     delivery_instructions: nil,\n...>     contact_number: \"+1-555-555-5555\",\n...>     last_verified: ~U[2025-01-10 17:14:04.738331Z]\n...>   },\n...>   created_at: ~U[2025-01-10 17:14:04.738331Z],\n...>   delivery_attempts: 3,\n...>   delivery_notes: [\"Leave at the front door\", \"Ring the bell\"],\n...>   document_reference:\n...>     {:ref, \"projects/my_project/databases/my_database/documents/collection1/doccument1\"}\n...> })\n{:ok,\n %PPlusFireStore.Model.Document{\n   path: \"projects/my_project/databases/my_database/documents/addresses/DxN4EvMyWCh7oTfVmxP9\",\n   data: %{\n     \"additional_info\" => %{\n       \"contact_number\" => \"+1-555-555-5555\",\n       \"delivery_instructions\" => nil,\n       \"is_residential\" => true,\n       \"last_verified\" => ~U[2025-01-10 17:14:04.738331Z]\n     },\n     \"city\" => \"Anytown\",\n     \"coordinates\" => %{\"latitude\" => 37.7749, \"longitude\" => -122.4194},\n     \"country\" => \"USA\",\n     \"created_at\" => ~U[2025-01-10 17:14:04.738331Z],\n     \"delivery_attempts\" => 3,\n     \"delivery_notes\" => [\"Leave at the front door\", \"Ring the bell\"],\n     \"document_reference\" => \"projects/my_project/databases/my_database/documents/collection1/doccument1\",\n     \"postal_code\" => \"12345\",\n     \"state\" => \"CA\",\n     \"street\" => \"123 Main St\"\n   },\n   created_at: ~U[2025-01-20 21:23:18.762968Z],\n   updated_at: ~U[2025-01-20 21:23:18.762968Z]\n}}\n```","ref":"readme.html#update-a-document"},{"type":"extras","title":"Delete a document - README","doc":"The `delete_document/2` function provides a simple way to delete your documents in Firestore.\n\nYour repository module is already configured with the base path and the database ID.\n\n`projects/ /databases/ /documents`\n\nSo you only need to specify the path within the collection.\n\n```elixir\niex> MyApp.MyFireStoreRepo.delete_document(\"addresses/DxN4EvMyWCh7oTfVmxP9\")\n{:ok, :deleted}\n```\n\nThe Google API returns `GoogleApi.Firestore.V1.Model.Empty{}` when a document is deleted, even if it does not exist or has already been deleted. This can cause some confusion, so by default `delete_document/1` sends the parameter `:\"currentDocument.exists\"` with the value `true`. This makes FireStore return an error if the document does not exist or has already been deleted.\n\nYou can pass the `:currentDocument.exists` parameter in the options to avoid this behavior.\n\n```elixir\nMyApp.MyFireStoreRepo.delete_document(\"addresses/DxN4EvMyWCh7oTfVmxP9\", [\"currentDocument.exists\": false])\n```","ref":"readme.html#delete-a-document"},{"type":"extras","title":"Contributing - README","doc":"","ref":"readme.html#contributing"},{"type":"extras","title":"Requirements - README","doc":"- Elixir 1.18+\n- Erlang 27+\n- Docker\n- Docker Compose","ref":"readme.html#requirements"},{"type":"extras","title":"Steps - README","doc":"1. Fork the project\n2. Create a feature branch\n3. Create a pull request","ref":"readme.html#steps"},{"type":"extras","title":"Tests - README","doc":"Run the firestore emulator container:\n\n```bash\ndocker compose up -d\n```\n\nRun tests:\n\n```bash\nmix test\n```\n\nRun Coverage, checks and tests:\n\n```bash\nmix ci\n```","ref":"readme.html#tests"},{"type":"extras","title":"Publishing - README","doc":"Pull requests format\n\nFeature\n\n```bash\ngit checkout -b feature/feature-name\n```\n\nfix\n\n```bash\ngit checkout -b fix/bugfix-name\n```\n\nCommit\n\n```bash\ngit commit -am \"feat: commit message\"\n```","ref":"readme.html#publishing"},{"type":"extras","title":"License - README","doc":"[MIT License](LICENSE)","ref":"readme.html#license"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2025 MedFlow\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}